# -*- coding: utf-8 -*-

__author__ = 'bprinty'
__email__ = 'bprinty@gmail.com'
__version__ = '0.0.4'


from .server import Server
from . import client as requests
from .client import get, post, put, delete, cache
